/**
 * @file
 * @author Timothy Redaelli
 */

#include "lista.h"
#include "freebsd.h"

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <err.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define PGDIR "/var/db/pkg"

/**
 * Return the file size or 0 if the file doesn't exist
 * @param path is the path of the file
 * @return The file size (or 0 if the file doesn't exist)
 */
static inline size_t filesize(char *path);

static inline char *mkpath(const char *a, const char *b, const char *c);

int pmodule(spackage **package) {
	DIR *dirp;
	FILE *filep;
	struct dirent *dp;
	int i;
	
	dirp = opendir(PGDIR);
	if(dirp == NULL)
		err(EXIT_FAILURE, "opendir");
	/** Read the directories */
	while((dp = readdir(dirp)) != NULL) {
		char *tmpfname;
		char *tmpdesc;
		size_t fsize;
		
		if(dp->d_name[0] == '.')
			continue;
		tmpfname = mkpath((const char*)PGDIR, dp->d_name, (const char*)"+REQUIRED_BY");
		if(filesize(tmpfname) != 0) {
			free(tmpfname);
			continue;
		}
		free(tmpfname);
		tmpfname = mkpath((const char*)PGDIR, dp->d_name, (const char*)"+DESC");
		if((fsize = filesize(tmpfname)) == 0) {
			free(tmpfname);
			continue;
		}
		filep = fopen(tmpfname, "r");
		free(tmpfname);
		tmpdesc = mymalloc((fsize + 1) * sizeof(char));
		for(i = 0; !feof(filep); i++) {
			tmpdesc[i] = fgetc(filep);
		}
		tmpdesc[i - 1] = '\0';
		fclose(filep);
		add_package(dp->d_name, tmpdesc, package);
		free(tmpdesc);
	}
	closedir(dirp);

	return 1;
}

int pdelete(spackage **package) {
	spackage *tmp = *package;
	int ret;
	for(; tmp != NULL; tmp=tmp->next) {
		if(tmp->toremove == 1) {
			char *cmd;
			cmd = mymalloc((strlen(tmp->name) + sizeof("pkg_delete") + 1) * sizeof(char));
			sprintf(cmd, "pkg_delete %s", tmp->name);
			ret = system(cmd);
			free(cmd);
		}
	}
	return ret;
}

static inline size_t filesize(char *path) {
	struct stat stp;
	size_t ret = 0;
	if(!((access(path, F_OK) == -1) || (stat(path, &stp) == -1)))
		ret = stp.st_size;
	return ret;
}

static inline char *mkpath(const char *a, const char *b, const char *c) {
	size_t lena = strlen(a), lenb = strlen(b), lenc = strlen(c);
	char *buff = mymalloc(lena + lenb + lenc + 3);
	size_t i, j = 0;
	for(i = 0; i < lena; i++, j++) 
		buff[j] = a[i];
	buff[j++] = '/';
	for(i = 0; i < lenb; i++, j++)
		buff[j] = b[i];
	buff[j++] = '/';
	for(i = 0; i < lenc; i++, j++)
		buff[j] = c[i];
	buff[j++]= '\0';
	return buff;
}
