/**
 * @file
 * @author Timothy Redaelli
 */

#include "lista.h"
#include "freebsd.h"
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

void printPackages(spackage **package);
void del_package(spackage *node, spackage **package);

void *mymalloc(size_t len) {
	void *tmp;
	tmp = malloc(len);
	if (tmp == NULL) {
		err(EXIT_FAILURE, NULL);
	}
	return tmp;
}

char *mystrdup(const char *s) {
	char *tmp;
	size_t len = (strlen(s) + 1) * sizeof(char);
	
	tmp = mymalloc(len);
	memcpy(tmp, s, len);

	return tmp;
}

int add_package(char *name, char *desc, spackage **package) {
	spackage *p;
	if (*package == NULL) {		/* Primo elemento */
		*package = mymalloc(sizeof(spackage));
		(*package)->name = mystrdup(name);
		(*package)->description = mystrdup(desc);
		(*package)->toremove = 0;
		(*package)->next = NULL;
		p = *package;
	} else {			/* Altro elemento */
		p = mymalloc(sizeof(spackage));
		p->name = mystrdup(name);
		p->description = mystrdup(desc);
		p->toremove = 0;
		p->next = *package;
		p->prev = NULL;
		(*package)->prev = p;
	}
	*package = p;
	return 1;
}

void clear_list(spackage **package) {
	spackage *tmp;
	
	while(*package != NULL) {
		tmp = *package;
		*package = (*package)->next;
		tmp->next = NULL;
		tmp->prev = NULL;
		free(tmp->name);
		free(tmp->description);
		free(tmp);
	}
}
#if 0
spackage *swap(spackage *current) {
	spackage *before = current->prev, *after = current->next, *tmp = after ? after->next : NULL;
	
	if(before)
		before->next = after;
	else
		head = after;
	if(after) {
		after->next = current;
		after->prev = before;
	}
	current->next = tmp;
	current->prev = after;
	if(tmp)
		tmp->prev = current;

	return after ? after : current;
}
#endif

void sortlist(spackage **package) {
	int sorted = 0;
	spackage *head = *package;
	
	while(!sorted) {
		*package = head;
		sorted = 1;
		while(*package != NULL && (*package)->next != NULL) {
			if(strcmp((*package)->name, (*package)->next->name) > 0) {
				spackage *before = (*package)->prev, *after = (*package)->next, *tmp = after ? after->next : NULL;
				sorted = 0;

				if(before)
					(before)->next = after;
				else
					head = after;
				if(after) {
					after->next = *package;
					after->prev = before;
				}
				(*package)->next = tmp;
				(*package)->prev = after;
				if(tmp)
					tmp->prev = (*package);
			}
			*package = (*package)->next;
		}
	}
	*package = head;
}

spackage *duplist(spackage **src) {
	spackage *package = NULL, *tmp;

	for(tmp = *src; tmp != NULL; tmp = tmp->next) {
		add_package(tmp->name, tmp->description, &package);
	}
	return package;
}

void difflists(spackage **before, spackage **after) {
	spackage *tmpa, *tmpb = *before;

	for(tmpb = *before; tmpb != NULL; tmpb = tmpb->next) {
		for(tmpa = *after; tmpa != NULL; tmpa = tmpa->next) {
			if(strcmp(tmpb->name, tmpa->name) == 0) {
				del_package(tmpa, after);
			}
		}
	}
}

void del_package(spackage *node, spackage **package) {
	spackage *before = node->prev;

	if(before != NULL) {
		before->next = node->next;
		if(node->next != NULL)
			node->next->prev = before;
	} else {
		*package = node->next;
	}

	node->next = NULL;
	node->prev = NULL;

	free(node->name);
	free(node->description);
	free(node);
}
