/**
 * @file
 * @author Timothy Redaelli
 */

#ifndef LISTA_H
#define LISTA_H

#include <stdlib.h>

/**
 * A package
 */
typedef struct pacchetto {
	char *name; /**< the name of the package */
	char *description; /**< the description of the package */
	int toremove; /**< set to 1 if you want to remove this package */
	struct pacchetto *next; /**< next package */
	struct pacchetto *prev; /**< previous package */
} spackage;

/**
 * Secure malloc
 * @param len is the size of the pointer
 * @return the pointer
 */
void *mymalloc(size_t len);
/**
 * duplicate a string
 * The mystrdup() function returns a pointer to a new string which is a duplicate of the string s. Memory for the new string is obtained with mymalloc, and can be freed with free.
 * @param s the string to duplicate
 * @return the string duplicated and allocated
 */
char *mystrdup(const char *s);
/**
 * Add a package in the list package
 * @param name is the name of the package
 * @param desc is the description of the package
 * @param package is the list where it will be add the package
 */
int add_package(char *name, char *desc, spackage **package);
/**
 * duplicate a list
 * @param src the list to duplicate
 * @return the list duplicated
 */
spackage *duplist(spackage **src);
/**
 * Sort the list by name
 * @param package is a pointer to the list to sort
 */
void sortlist(spackage **package);
/**
 * Compare 2 list deleting all the elements presents in before in after
 * @param before the first list
 * @param after the second list
 */
void difflists(spackage **before, spackage **after);
/**
 * Delete all the elements of a list
 * @param package the list to delete
 */
void clear_list(spackage **package);

#endif
